from typing import BinaryIO

import ffmpeg

from ._base import BaseStep


class Mp4ToWebpStep(BaseStep):
    def __init__(
        self,
        input_file: BinaryIO,
        output_file: BinaryIO,
        *,
        width: int,
        frame_rate: int,
    ) -> None:
        self._input_file = input_file
        self._output_file = output_file
        self._width = width
        self._frame_rate = frame_rate

    async def run(self) -> None:
        (
            ffmpeg
            .input(self._input_file.name)
            .filter('scale', self._width, -1)
            .output(
                self._output_file.name,
                vcodec='libwebp',  # use the WebP encoder
                loop=0,  # 0 = infinite loop
                lossless=0,  # 1 = lossless; 0 = lossy
                compression_level=6,  # 0–6 (higher = slower but better)
                **{'qscale:v': 50},  # quality: 0 (best)–100 (worst)
                r=self._frame_rate,  # output framerate
            )
            .overwrite_output()
            .run()
        )



from typing import BinaryIO

from PIL import Image

from ._base import BaseStep


class JpgToWebpStep(BaseStep):
    def __init__(
        self,
        input_file: BinaryIO,
        output_file: BinaryIO,
        *,
        width: int,
        quality: int,
    ) -> None:
        self._input_file = input_file
        self._output_file = output_file
        self._width = width
        self._quality = quality

    async def run(self) -> None:
        with Image.open(self._input_file.name) as image:
            image = image.convert('RGB')
            width, height = image.size
            new_width = self._width
            new_height = int(height * (new_width / width))
            resized = image.resize((new_width, new_height), Image.LANCZOS)
            resized.save(
                self._output_file,
                format='WEBP',
                lossless=False,
                quality=self._quality,
            )



from typing import BinaryIO

import httpx

from ._base import BaseStep


class DownloadStep(BaseStep):
    def __init__(
        self,
        file: BinaryIO,
        *,
        url: str,
    ) -> None:
        self._file = file
        self._url = url

    async def run(self) -> None:
        async with httpx.AsyncClient() as http_client:
            response = await http_client.get(self._url)
            self._file.write(response.content)



from typing import BinaryIO, Literal

from aiobotocore.session import AioSession, get_session

from ._base import BaseStep


class AwsSession:
    _session: AioSession | None = None

    @classmethod
    def get(cls) -> AioSession:
        if cls._session is None:
            cls._session = get_session()

        return cls._session


class UploadToS3Step(BaseStep):
    def __init__(
        self,
        file: BinaryIO,
        *,
        aws_region: str,
        aws_access_key_id: str,
        aws_secret_access_key: str,
        aws_s3_bucket: str,
        aws_s3_object_key: str,
        content_type: Literal['image/webp'],
    ) -> None:
        self._file = file
        self._aws_region = aws_region
        self._aws_access_key_id = aws_access_key_id
        self._aws_secret_access_key = aws_secret_access_key
        self._aws_s3_bucket = aws_s3_bucket
        self._aws_s3_object_key = aws_s3_object_key
        self._content_type = content_type

    async def run(self) -> None:
        aws_session = AwsSession.get()

        async with aws_session.create_client(
            's3',
            region_name=self._aws_region,
            aws_access_key_id=self._aws_access_key_id,
            aws_secret_access_key=self._aws_secret_access_key,
        ) as s3_client:
            await s3_client.put_object(
                Bucket=self._aws_s3_bucket,
                Key=self._aws_s3_object_key,
                Body=self._file,
                ContentType=self._content_type,
            )
